@@ -1,6 +1,7 @@
 /*
-  Copyright (C) 2010 The ESPResSo project
-  Copyright (C) 2002,2003,2004,2005,2006,2007,2008,2009,2010 Max-Planck-Institute for Polymer Research, Theory Group, PO Box 3148, 55021 Mainz, Germany
+  Copyright (C) 2010,2012 The ESPResSo project
+  Copyright (C) 2002,2003,2004,2005,2006,2007,2008,2009,2010 
+    Max-Planck-Institute for Polymer Research, Theory Group
   
   This file is part of ESPResSo.
   
@@ -246,7 +247,7 @@ void distribute(int size)
 {
   double send_buf[8];
   copy_vec(send_buf, gblcblk, size);
-  MPI_Allreduce(send_buf, gblcblk, size, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);
+  MPI_Allreduce(send_buf, gblcblk, size, MPI_DOUBLE, MPI_SUM, comm_cart);
 }
 
 #ifdef CHECKPOINTS
@@ -1186,7 +1187,7 @@ int ELC_tune(double error)
   }
 
   if (h < 0)
-    return TCL_ERROR;
+    return ES_ERROR;
 
   elc_params.far_cut = min_inv_boxl;
   do {
@@ -1201,99 +1202,17 @@ int ELC_tune(double error)
   }
   while (err > error && elc_params.far_cut < MAXIMAL_FAR_CUT);
   if (elc_params.far_cut >= MAXIMAL_FAR_CUT)
-    return TCL_ERROR;
+    return ES_ERROR;
   elc_params.far_cut -= min_inv_boxl;
   elc_params.far_cut2 = SQR(elc_params.far_cut);
 
-  return TCL_OK;
+  return ES_OK;
 }
 
 /****************************************
  * COMMON PARTS
  ****************************************/
 
-int tclprint_to_result_ELC(Tcl_Interp *interp)
-{
-  char buffer[TCL_DOUBLE_SPACE];
-
-  Tcl_PrintDouble(interp, elc_params.maxPWerror, buffer);
-  Tcl_AppendResult(interp, "} {coulomb elc ", buffer, (char *) NULL);
-  Tcl_PrintDouble(interp, elc_params.gap_size, buffer);
-  Tcl_AppendResult(interp, " ", buffer, (char *) NULL);
-  Tcl_PrintDouble(interp, elc_params.far_cut, buffer);
-  Tcl_AppendResult(interp, " ", buffer, (char *) NULL);
-  if (!elc_params.neutralize)
-    Tcl_AppendResult(interp, " noneutralization", (char *) NULL);
-  if (elc_params.dielectric_contrast_on) {
-    Tcl_PrintDouble(interp, elc_params.di_top, buffer);
-    Tcl_AppendResult(interp, " dielectric ", buffer, (char *) NULL);
-    Tcl_PrintDouble(interp, elc_params.di_mid, buffer);
-    Tcl_AppendResult(interp, " ", buffer, (char *) NULL);
-    Tcl_PrintDouble(interp, elc_params.di_bot, buffer);
-    Tcl_AppendResult(interp, " ", buffer, (char *) NULL);
-    Tcl_PrintDouble(interp, elc_params.space_layer, buffer);
-    Tcl_AppendResult(interp, " ", buffer, (char *) NULL);
-  }
-  return TCL_OK;
-}
-
-int tclcommand_inter_coulomb_parse_elc_params(Tcl_Interp * interp, int argc, char ** argv)
-{
-  double pwerror;
-  double gap_size;
-  double far_cut = -1;
-  double top = 1, mid = 1, bot = 1;
-  int neutralize = 1;
-
-  if (argc < 2) {
-    Tcl_AppendResult(interp, "either nothing or elc <pwerror> <minimal layer distance> {<cutoff>}  {dielectric <di_top> <di_mid> <di_bottom>} {noneutralization} expected, not \"",
-		     argv[0], "\"", (char *)NULL);
-    return TCL_ERROR;
-  }
-  if (!ARG0_IS_D(pwerror))
-    return TCL_ERROR;
-  if (!ARG1_IS_D(gap_size))
-    return TCL_ERROR;
-
-  argc -= 2; argv += 2;
-
-  if (argc > 0) {
-    // if there, parse away manual cutoff
-    if(ARG0_IS_D(far_cut)) {
-      argc--; argv++;
-    }
-    else
-      Tcl_ResetResult(interp);
-
-    while (argc > 0) {
-      if (ARG0_IS_S("noneutralization") || ARG0_IS_S("-noneutralization")) {
-	neutralize = 0;
-	argc--; argv++;
-      }
-      else if (argc >= 4 && ARG0_IS_S("dielectric")) {
-	// just a dummy, not used, as it is only printed for information
-	// purposes. We need to calculate it
-	double space_layer_dummy;
-
-	if (!ARG_IS_D(1,top) || !ARG_IS_D(2,mid) || !ARG_IS_D(3,bot))
-	  return TCL_ERROR;
-	argc -= 4; argv += 4;
-
-	if (argc > 0 && ARG_IS_D(4, space_layer_dummy)) {
-	  argc--; argv++;
-	}
-      }
-      else {
-	Tcl_AppendResult(interp, "either nothing or elc <pwerror> <minimal layer distance> {<cutoff>}  {dielectric <di_top> <di_mid> <di_bottom>} {noneutralization} expected, not \"",
-			 argv[0], "\"", (char *)NULL);
-	return TCL_ERROR;
-      }
-    }
-  }
-  CHECK_VALUE(ELC_set_params(pwerror, gap_size, far_cut, neutralize, top, mid, bot),
-	      "choose a 3d electrostatics method prior to ELC");
-}
-
 int ELC_sanity_checks()
 {
   char *errtxt;
@@ -1318,7 +1237,7 @@ void ELC_init()
     elc_params.space_layer = (1./3.)*elc_params.gap_size;
     // but make sure we leave enough space to not have to bother with overlapping
     // realspace P3M
-    maxsl = elc_params.gap_size - p3m.r_cut;
+    maxsl = elc_params.gap_size - p3m.params.r_cut;
     // and make sure the space layer is not bigger than half the actual simulation box,
     // to avoid overlaps
     if (maxsl > .5*elc_params.h) maxsl = .5*elc_params.h;
@@ -1339,14 +1258,14 @@ void ELC_init()
 
   if (elc_params.far_calculated &&
       (coulomb.method == COULOMB_ELC_P3M && elc_params.dielectric_contrast_on)) {
-    if (ELC_tune(elc_params.maxPWerror) == TCL_ERROR) {
+    if (ELC_tune(elc_params.maxPWerror) == ES_ERROR) {
       errtxt = runtime_error(128);
       ERROR_SPRINTF(errtxt, "{008 ELC auto-retuning failed, gap size too small} ");
     }
   }
   if (coulomb.method == COULOMB_ELC_P3M && elc_params.dielectric_contrast_on) {
-    p3m.additional_mesh[0] = p3m.additional_mesh[1] = 0;
-    p3m.additional_mesh[2] = elc_params.space_layer; 
+    p3m.params.additional_mesh[0] = p3m.params.additional_mesh[1] = 0;
+    p3m.params.additional_mesh[2] = elc_params.space_layer; 
   }
 }
 
@@ -1361,11 +1280,6 @@ void ELC_on_resort_particles()
   partblk   = realloc(partblk,  n_localpart*8*sizeof(double));
 }
 
-/* TODO: This function is not used anywhere :) To be removed?  */
-void ELC_on_coulomb_change()
-{
-}
-
 int ELC_set_params(double maxPWerror, double gap_size, double far_cut, int neutralize,
 		   double top, double mid, double bot)
 {
@@ -1406,11 +1320,11 @@ int ELC_set_params(double maxPWerror, double gap_size, double far_cut, int neutr
   switch (coulomb.method) {
   case COULOMB_ELC_P3M:
   case COULOMB_P3M:
-    p3m.epsilon = P3M_EPSILON_METALLIC;
+    p3m.params.epsilon = P3M_EPSILON_METALLIC;
     coulomb.method = COULOMB_ELC_P3M;
     break;
   default:
-    return TCL_ERROR;
+    return ES_ERROR;
   }
 
   elc_params.far_cut = far_cut;
@@ -1420,14 +1334,14 @@ int ELC_set_params(double maxPWerror, double gap_size, double far_cut, int neutr
   }
   else {
     elc_params.far_calculated = 1;
-    if (ELC_tune(elc_params.maxPWerror) == TCL_ERROR) {
+    if (ELC_tune(elc_params.maxPWerror) == ES_ERROR) {
       char *errtxt = runtime_error(128);
       ERROR_SPRINTF(errtxt, "{009 ELC tuning failed, gap size too small} ");
     }
   }
   mpi_bcast_coulomb_params();
 
-  return TCL_OK;
+  return ES_OK;
 }
 
 ////////////////////////////////////////////////////////////////////////////////////
@@ -1451,7 +1365,7 @@ void ELC_P3M_self_forces()
 	get_mi_vector(d, p[i].r.p, pos);
 	dist2 = sqrlen(d);
 	dist = sqrt(dist2);
-	add_p3m_coulomb_pair_force(q,d,dist2,dist,p[i].f.f);
+	p3m_add_pair_force(q,d,dist2,dist,p[i].f.f);
       }
       if(p[i].r.p[2]>(elc_params.h-elc_params.space_layer)) {
 	q=elc_params.di_mid_top*p[i].p.q*p[i].p.q;
@@ -1459,7 +1373,7 @@ void ELC_P3M_self_forces()
 	get_mi_vector(d, p[i].r.p, pos);
 	dist2 = sqrlen(d);
 	dist = sqrt(dist2);
-	add_p3m_coulomb_pair_force(q,d,dist2,dist,p[i].f.f);
+	p3m_add_pair_force(q,d,dist2,dist,p[i].f.f);
       }
     }
   }
@@ -1467,7 +1381,7 @@ void ELC_P3M_self_forces()
 
 ////////////////////////////////////////////////////////////////////////////////////
 
-void ELC_P3M_charge_assign_both()
+void ELC_p3m_charge_assign_both()
 {
   Cell *cell;
   Particle *p;
@@ -1476,7 +1390,7 @@ void ELC_P3M_charge_assign_both()
   /* charged particle counter, charge fraction counter */
   int cp_cnt=0;
   /* prepare local FFT mesh */
-  for(i=0; i<lm.size; i++) rs_mesh[i] = 0.0;
+  for(i=0; i<p3m.local_mesh.size; i++) p3m.rs_mesh[i] = 0.0;
 
   for (c = 0; c < local_cells.n; c++) {
     cell = local_cells.cell[c];
@@ -1484,35 +1398,37 @@ void ELC_P3M_charge_assign_both()
     np = cell->n;
     for(i = 0; i < np; i++) {
       if( p[i].p.q != 0.0 ) {
-	P3M_assign_charge(p[i].p.q, p[i].r.p,cp_cnt);
+	p3m_assign_charge(p[i].p.q, p[i].r.p,cp_cnt);
 
 	if(p[i].r.p[2]<elc_params.space_layer) {
 	  double q=elc_params.di_mid_bot*p[i].p.q;
 	  pos[0]=p[i].r.p[0]; pos[1]=p[i].r.p[1]; pos[2]=-p[i].r.p[2];
-	  P3M_assign_charge(q, pos, -1);
+	  p3m_assign_charge(q, pos, -1);
 	}
 	
 	if(p[i].r.p[2]>(elc_params.h-elc_params.space_layer)) {
 	  double q=elc_params.di_mid_top*p[i].p.q;
 	  pos[0]=p[i].r.p[0]; pos[1]=p[i].r.p[1]; pos[2]=2*elc_params.h-p[i].r.p[2];
-	  P3M_assign_charge(q, pos,-1);
+	  p3m_assign_charge(q, pos,-1);
 	}
 
 	cp_cnt++;
       }
     }
   }
-  P3M_shrink_wrap_charge_grid(cp_cnt);
+#ifdef P3M_STORE_CA_FRAC
+  p3m_shrink_wrap_charge_grid(cp_cnt);
+#endif
 }
 
-void ELC_P3M_charge_assign_image() 
+void ELC_p3m_charge_assign_image() 
 {
   Cell *cell;
   Particle *p;
   double pos[3];
   int i,c,np;
   /* prepare local FFT mesh */
-  for(i=0; i<lm.size; i++) rs_mesh[i] = 0.0;
+  for(i=0; i<p3m.local_mesh.size; i++) p3m.rs_mesh[i] = 0.0;
 
   for (c = 0; c < local_cells.n; c++) {
     cell = local_cells.cell[c];
@@ -1524,13 +1440,13 @@ void ELC_P3M_charge_assign_image()
 	if(p[i].r.p[2]<elc_params.space_layer) {
 	  double q=elc_params.di_mid_bot*p[i].p.q;
 	  pos[0]=p[i].r.p[0]; pos[1]=p[i].r.p[1]; pos[2]=-p[i].r.p[2];
-	  P3M_assign_charge(q, pos,-1);
+	  p3m_assign_charge(q, pos,-1);
 	}
 	
 	if(p[i].r.p[2]>(elc_params.h-elc_params.space_layer)) {
 	  double q=elc_params.di_mid_top*p[i].p.q;
 	  pos[0]=p[i].r.p[0]; pos[1]=p[i].r.p[1]; pos[2]=2*elc_params.h-p[i].r.p[2];
-	  P3M_assign_charge(q, pos, -1);
+	  p3m_assign_charge(q, pos, -1);
 	}
       }
     }
@@ -1553,7 +1469,7 @@ void ELC_P3M_dielectric_layers_force_contribution(Particle *p1, Particle *p2, do
     get_mi_vector(d, p2->r.p, pos);
     dist2 = sqrlen(d);
     dist = sqrt(dist2);
-    add_p3m_coulomb_pair_force(q,d,dist2,dist,force2);
+    p3m_add_pair_force(q,d,dist2,dist,force2);
   } 
   
   if(p1->r.p[2]>(elc_params.h-elc_params.space_layer)) {
@@ -1562,7 +1478,7 @@ void ELC_P3M_dielectric_layers_force_contribution(Particle *p1, Particle *p2, do
     get_mi_vector(d, p2->r.p, pos);
     dist2 = sqrlen(d);
     dist = sqrt(dist2);
-    add_p3m_coulomb_pair_force(q,d,dist2,dist,force2);
+    p3m_add_pair_force(q,d,dist2,dist,force2);
   } 
   
   if(tp2<elc_params.space_layer) {
@@ -1571,7 +1487,7 @@ void ELC_P3M_dielectric_layers_force_contribution(Particle *p1, Particle *p2, do
     get_mi_vector(d, p1->r.p, pos);
     dist2 = sqrlen(d);
     dist = sqrt(dist2);
-    add_p3m_coulomb_pair_force(q,d,dist2,dist,force1);
+    p3m_add_pair_force(q,d,dist2,dist,force1);
   }
   
   if(tp2>(elc_params.h-elc_params.space_layer)) {	  
@@ -1580,7 +1496,7 @@ void ELC_P3M_dielectric_layers_force_contribution(Particle *p1, Particle *p2, do
     get_mi_vector(d, p1->r.p, pos);
     dist2 = sqrlen(d);
     dist = sqrt(dist2);
-    add_p3m_coulomb_pair_force(q,d,dist2,dist,force1);
+    p3m_add_pair_force(q,d,dist2,dist,force1);
   }
 }
 
@@ -1602,7 +1518,7 @@ double ELC_P3M_dielectric_layers_energy_contribution(Particle *p1, Particle *p2)
     get_mi_vector(d, p2->r.p, pos);
     dist2 = sqrlen(d);
     dist = sqrt(dist2);
-    eng+=p3m_coulomb_pair_energy(q,d,dist2,dist);
+    eng+=p3m_pair_energy(q,d,dist2,dist);
     
   } 
   
@@ -1612,7 +1528,7 @@ double ELC_P3M_dielectric_layers_energy_contribution(Particle *p1, Particle *p2)
     get_mi_vector(d, p2->r.p, pos);
     dist2 = sqrlen(d);
     dist = sqrt(dist2);
-    eng+=p3m_coulomb_pair_energy(q,d,dist2,dist);
+    eng+=p3m_pair_energy(q,d,dist2,dist);
   } 
   
   if(tp2<elc_params.space_layer) {
@@ -1621,7 +1537,7 @@ double ELC_P3M_dielectric_layers_energy_contribution(Particle *p1, Particle *p2)
     get_mi_vector(d, p1->r.p, pos);
     dist2 = sqrlen(d);
     dist = sqrt(dist2);
-    eng+=p3m_coulomb_pair_energy(q,d,dist2,dist);
+    eng+=p3m_pair_energy(q,d,dist2,dist);
   }
   
   if(tp2>(elc_params.h-elc_params.space_layer)) {
@@ -1630,7 +1546,7 @@ double ELC_P3M_dielectric_layers_energy_contribution(Particle *p1, Particle *p2)
     get_mi_vector(d, p1->r.p, pos);
     dist2 = sqrlen(d);
     dist = sqrt(dist2);
-    eng+=p3m_coulomb_pair_energy(q,d,dist2,dist);
+    eng+=p3m_pair_energy(q,d,dist2,dist);
   }
 
   // fprintf(stderr,"energy is %f\n",eng);
@@ -1661,7 +1577,7 @@ double ELC_P3M_dielectric_layers_energy_self() {
 	get_mi_vector(d, p1[i].r.p, pos);
 	dist2 = sqrlen(d);
 	dist = sqrt(dist2);
-	eng+=p3m_coulomb_pair_energy(q,d,dist2,dist);
+	eng+=p3m_pair_energy(q,d,dist2,dist);
 	//	fprintf(stderr,"energy is %f\n",eng);
       }
 	
@@ -1671,7 +1587,7 @@ double ELC_P3M_dielectric_layers_energy_self() {
 	get_mi_vector(d, p1[i].r.p, pos);
 	dist2 = sqrlen(d);
 	dist = sqrt(dist2);
-       	eng+=p3m_coulomb_pair_energy(q,d,dist2,dist);
+       	eng+=p3m_pair_energy(q,d,dist2,dist);
 	//	fprintf(stderr,"energy is %f\n",eng);
       }
     }
@@ -1716,10 +1632,10 @@ void  ELC_P3M_modify_p3m_sums_both()
     }
   }
   
-  MPI_Allreduce(node_sums, tot_sums, 3, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);
-  p3m_sum_qpart    += (int)(tot_sums[0]+0.1);
-  p3m_sum_q2       += tot_sums[1];
-  p3m_square_sum_q += SQR(tot_sums[2]);
+  MPI_Allreduce(node_sums, tot_sums, 3, MPI_DOUBLE, MPI_SUM, comm_cart);
+  p3m.sum_qpart    += (int)(tot_sums[0]+0.1);
+  p3m.sum_q2       += tot_sums[1];
+  p3m.square_sum_q += SQR(tot_sums[2]);
 }
 
 void  ELC_P3M_modify_p3m_sums_image()
@@ -1757,10 +1673,11 @@ void  ELC_P3M_modify_p3m_sums_image()
     }
   }
   
-  MPI_Allreduce(node_sums, tot_sums, 3, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);
-  p3m_sum_qpart    = (int)(tot_sums[0]+0.1);
-  p3m_sum_q2       = tot_sums[1];
-  p3m_square_sum_q = SQR(tot_sums[2]);
+  MPI_Allreduce(node_sums, tot_sums, 3, MPI_DOUBLE, MPI_SUM, comm_cart);
+
+  p3m.sum_qpart    = (int)(tot_sums[0]+0.1);
+  p3m.sum_q2       = tot_sums[1];
+  p3m.square_sum_q = SQR(tot_sums[2]);
 }
 
 // this function is required in force.c for energy evaluation
@@ -1799,10 +1716,11 @@ void  ELC_P3M_restore_p3m_sums()
     }
   }
   
-  MPI_Allreduce(node_sums, tot_sums, 3, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);
-  p3m_sum_qpart    -= (int)(tot_sums[0]+0.1);
-  p3m_sum_q2       -= tot_sums[1];
-  p3m_square_sum_q -= SQR(tot_sums[2]);
+  MPI_Allreduce(node_sums, tot_sums, 3, MPI_DOUBLE, MPI_SUM, comm_cart);
+
+  p3m.sum_qpart    -= (int)(tot_sums[0]+0.1);
+  p3m.sum_q2       -= tot_sums[1];
+  p3m.square_sum_q -= SQR(tot_sums[2]);
 }
 
 #endif
