@@ -1,18 +1,21 @@
-/* $Id: lbgpu.h $
- *
- * This file is part of the ESPResSo distribution (http://www.espresso.mpg.de).
- * It is therefore subject to the ESPResSo license agreement which you
- * accepted upon receiving the distribution and by which you are
- * legally bound while utilizing this file in any form or way.
- * There is NO WARRANTY, not even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * You should have received a copy of that license along with this
- * program; if not, refer to http://www.espresso.mpg.de/license.html
- * where its current version can be found, or write to
- * Max-Planck-Institute for Polymer Research, Theory Group,
- * PO Box 3148, 55021 Mainz, Germany.
- * Copyright (c) 2002-2007; all rights reserved unless otherwise stated.
- */
+/* 
+   Copyright (C) 2010,2011,2012 The ESPResSo project
+
+   This file is part of ESPResSo.
+  
+   ESPResSo is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+   
+   ESPResSo is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+   
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 /** \file lbgpu.h
  * Header file for lbgpu.c
@@ -20,16 +23,10 @@
  * This is the header file for the Lattice Boltzmann implementation in lbgpu_cfile.c
  */
 
-
-
-
 #ifndef LB_GPU_H
 #define LB_GPU_H
 
-#include <tcl.h>
 #include "utils.h"
-//#include "lattice.h"
-
 #ifdef LB_GPU
 
 /* For the D3Q19 model most functions have a separate implementation
@@ -55,7 +52,6 @@
 #endif
 /*@}*/
 
-
 /**-------------------------------------------------------------------------*/
 /** Data structure holding the parameters for the Lattice Boltzmann system for gpu. */
 typedef struct {
@@ -118,18 +114,6 @@ typedef struct {
 
 } LB_parameters_gpu;
 
-/** Data structure holding the velocitydensities for the Lattice Boltzmann system. */
-typedef struct {
-
-  /** velocitydensity of the node */
-  float *vd;
-  /** seed for the random gen */
-  unsigned int *seed;
-  /** flag indicating whether this site belongs to a boundary */
-  unsigned int *boundary;
-
-} LB_nodes_gpu;
-
 /** Data structure holding the phys. values for the Lattice Boltzmann system. */
 typedef struct {
 
@@ -145,6 +129,19 @@ typedef struct {
 
 } LB_values_gpu;
 
+/** Data structure holding the velocitydensities for the Lattice Boltzmann system. */
+typedef struct {
+
+  /** velocitydensity of the node */
+  float *vd;
+  /** seed for the random gen */
+  unsigned int *seed;
+  /** flag indicating whether this site belongs to a boundary */
+  unsigned int *boundary;
+
+} LB_nodes_gpu;
+
+
 /** Data structure for the randomnr and the seed. */
 typedef struct {
 
@@ -192,40 +189,41 @@ typedef struct {
 
 } LB_particle_seed_gpu;
 
+void on_lb_params_change_gpu(int field);
+
+/************************************************************/
+/** \name Exported Variables */
+/************************************************************/
+/*@{*/
+
+/** 
+ */
 #ifdef __cplusplus
 extern "C" {
 #endif
-
-extern LB_parameters_gpu lbpar_gpu;
-
 /** Switch indicating momentum exchange between particles and fluid */
+extern LB_parameters_gpu lbpar_gpu;
+extern LB_values_gpu *host_values;
 extern int transfer_momentum_gpu;
-
 extern LB_extern_nodeforce_gpu *extern_nodeforces_gpu;
-extern int n_lb_boundaries_gpu;
+extern int n_lb_boundaries;
 
 #ifdef __cplusplus
 }
 #endif
-/** Eigenvalue of collision operator corresponding to shear viscosity. */
-//extern double lblambda;
 
-/** Eigenvalue of collision operator corresponding to bulk viscosity. */
-//extern double lblambda_bulk;
+
+/*@}*/
 
 /************************************************************/
 /** \name Exported Functions */
 /************************************************************/
 /*@{*/
 
-/** Updates the Lattice Boltzmann system for one time step.
- * This function performs the collision step and the streaming step.
- * If external forces are present, they are applied prior to the collisions.
- * If boundaries are present, it also applies the boundary conditions.
- */
 #ifdef __cplusplus
 extern "C" {
 #endif
+
 void lattice_boltzmann_update_gpu();
 
 /** (Pre-)initializes data structures. */
@@ -250,7 +248,6 @@ void lb_reinit_fluid_gpu();
 /** (Re-)initializes the particle array*/
 void lb_realloc_particles_gpu();
 
-
 void lb_init_GPU(LB_parameters_gpu *lbpar_gpu);
 void lb_integrate_GPU();
 void lb_particle_GPU(LB_particle_gpu *host_data);
@@ -259,30 +256,33 @@ void lb_get_values_GPU(LB_values_gpu *host_values);
 void lb_realloc_particle_GPU(LB_parameters_gpu *lbpar_gpu, LB_particle_gpu **host_data);
 void lb_copy_forces_GPU(LB_particle_force_gpu *host_forces);
 void lb_print_node_GPU(int single_nodeindex, LB_values_gpu *host_print_values);
+#ifdef LB_BOUNDARIES_GPU
 void lb_init_boundaries_GPU(int number_of_boundnodes, int *host_boundindex);
+#endif
 void lb_init_extern_nodeforces_GPU(int n_extern_nodeforces, LB_extern_nodeforce_gpu *host_extern_nodeforces, LB_parameters_gpu *lbpar_gpu);
 
 void lb_calc_particle_lattice_ia_gpu();
 void lb_send_forces_gpu();
-void calc_fluid_momentum_GPU(double* mom);
-void calc_fluid_temperature_GPU(double* cpu_temp);
+
+void lb_calc_fluid_mass_GPU(double* mass);
+void lb_calc_fluid_momentum_GPU(double* host_mom);
+void lb_calc_fluid_temperature_GPU(double* host_temp);
+void lb_get_boundary_flag_GPU(int single_nodeindex, unsigned int* host_flag);
+void lb_get_boundary_flags_GPU(unsigned int* host_bound_array);
+
+void lb_set_node_velocity_GPU(int single_nodeindex, float* host_velocity);
+
 void reinit_parameters_GPU(LB_parameters_gpu *lbpar_gpu);
 void lb_reinit_extern_nodeforce_GPU(LB_parameters_gpu *lbpar_gpu);
 void lb_reinit_GPU(LB_parameters_gpu *lbpar_gpu);
+int lb_lbnode_set_extforce_GPU(int ind[3], double f[3]);
+
 #ifdef __cplusplus
 }
 #endif
-void on_lb_params_change_gpu(int field);
-/** Parser for the TCL command lbnode. */
-int tclcommand_lbnode_gpu(Tcl_Interp *interp, int argc, char **argv);
 
-/** Parser for the TCL command \ref lbfluid. */
-int tclcommand_lbfluid_gpu(Tcl_Interp *interp, int argc, char **argv);
+/*@{*/
 
-int tclcommand_lbnode_extforce_gpu(ClientData data, Tcl_Interp *interp, int argc, char **argv);
-
-int tclcommand_lbprint_gpu(ClientData data, Tcl_Interp *interp, int argc, char **argv);
 #endif /* LB_GPU */
-#endif /* LB_GPU_H */
 
-/*@}*/
+#endif /* LB_GPU_H */
